@@ -1,5 +1,3 @@
-/* $Id: tiffsplit.c,v 1.14.2.5 2010-12-11 19:16:26 faxguy Exp $ */
-
 /*
  * Copyright (c) 1992-1997 Sam Leffler
  * Copyright (c) 1992-1997 Silicon Graphics, Inc.
@@ -25,6 +23,7 @@
  */
 
 #include "tif_config.h"
+#include "libport.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,8 +31,11 @@
 
 #include "tiffio.h"
 
-#ifndef HAVE_GETOPT
-extern int getopt(int, char**, char*);
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#endif
+#ifndef EXIT_FAILURE
+#define EXIT_FAILURE 1
 #endif
 
 #define	CopyField(tag, v) \
@@ -59,40 +61,46 @@ main(int argc, char* argv[])
 {
 	TIFF *in, *out;
 
-	if (argc < 2) {
-                fprintf(stderr, "%s\n\n", TIFFGetVersion());
-		fprintf(stderr, "usage: tiffsplit input.tif [prefix]\n");
-		return (-3);
+		if (argc < 2 || argc > 3) {
+			fprintf(stderr, "%s\n\n", TIFFGetVersion());
+			fprintf(stderr, "Split a multi-image TIFF into single-image TIFF files\n\n");
+			fprintf(stderr, "usage: tiffsplit input.tif [prefix]\n");
+		return (EXIT_FAILURE);
 	}
 	if (argc > 2) {
 		strncpy(fname, argv[2], sizeof(fname));
 		fname[sizeof(fname) - 1] = '\0';
 	}
+
 	in = TIFFOpen(argv[1], "r");
-	if (in != NULL) {
-		do {
-			size_t path_len;
-			char *path;
-			
-			newfilename();
+        if (in == NULL) {
+                return EXIT_FAILURE;
+        }
 
-			path_len = strlen(fname) + sizeof(TIFF_SUFFIX);
-			path = (char *) _TIFFmalloc(path_len);
-			strncpy(path, fname, path_len);
-			path[path_len - 1] = '\0';
-			strncat(path, TIFF_SUFFIX, path_len - strlen(path) - 1);
-			out = TIFFOpen(path, TIFFIsBigEndian(in)?"wb":"wl");
-			_TIFFfree(path);
+        do {
+                size_t path_len;
+                char *path;
 
-			if (out == NULL)
-				return (-2);
-			if (!tiffcp(in, out))
-				return (-1);
-			TIFFClose(out);
-		} while (TIFFReadDirectory(in));
-		(void) TIFFClose(in);
-	}
-	return (0);
+                newfilename();
+
+                path_len = strlen(fname) + sizeof(TIFF_SUFFIX);
+                path = (char *) _TIFFmalloc(path_len);
+                strncpy(path, fname, path_len);
+                path[path_len - 1] = '\0';
+                strncat(path, TIFF_SUFFIX, path_len - strlen(path) - 1);
+                out = TIFFOpen(path, TIFFIsBigEndian(in)?"wb":"wl");
+                _TIFFfree(path);
+
+                if (out == NULL)
+                        return (EXIT_FAILURE);
+                if (!tiffcp(in, out))
+                        return (EXIT_FAILURE);
+                TIFFClose(out);
+        } while (TIFFReadDirectory(in));
+
+        (void) TIFFClose(in);
+
+        return (EXIT_SUCCESS);
 }
 
 static void
@@ -119,7 +127,7 @@ newfilename(void)
 	if (fnum == MAXFILES) {
 		if (!defname || fname[0] == 'z') {
 			fprintf(stderr, "tiffsplit: too many files.\n");
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 		fname[0]++;
 		fnum = 0;
@@ -157,11 +165,11 @@ newfilename(void)
 static int
 tiffcp(TIFF* in, TIFF* out)
 {
-	uint16 bitspersample, samplesperpixel, compression, shortv, *shortav;
-	uint32 w, l;
+	uint16_t bitspersample, samplesperpixel, compression, shortv, *shortav;
+	uint32_t w, l;
 	float floatv;
 	char *stringv;
-	uint32 longv;
+	uint32_t longv;
 
 	CopyField(TIFFTAG_SUBFILETYPE, longv);
 	CopyField(TIFFTAG_TILEWIDTH, w);
@@ -172,7 +180,7 @@ tiffcp(TIFF* in, TIFF* out)
 	CopyField(TIFFTAG_SAMPLESPERPIXEL, samplesperpixel);
 	CopyField(TIFFTAG_COMPRESSION, compression);
 	if (compression == COMPRESSION_JPEG) {
-		uint16 count = 0;
+		uint32_t count = 0;
 		void *table = NULL;
 		if (TIFFGetField(in, TIFFTAG_JPEGTABLES, &count, &table)
 		    && count > 0 && table) {
@@ -199,10 +207,10 @@ tiffcp(TIFF* in, TIFF* out)
 	CopyField(TIFFTAG_TILEDEPTH, longv);
 	CopyField(TIFFTAG_SAMPLEFORMAT, shortv);
 	CopyField2(TIFFTAG_EXTRASAMPLES, shortv, shortav);
-	{ uint16 *red, *green, *blue;
+	{ uint16_t *red, *green, *blue;
 	  CopyField3(TIFFTAG_COLORMAP, red, green, blue);
 	}
-	{ uint16 shortv2;
+	{ uint16_t shortv2;
 	  CopyField2(TIFFTAG_PAGENUMBER, shortv, shortv2);
 	}
 	CopyField(TIFFTAG_ARTIST, stringv);
@@ -230,26 +238,27 @@ tiffcp(TIFF* in, TIFF* out)
 static int
 cpStrips(TIFF* in, TIFF* out)
 {
-	tsize_t bufsize  = TIFFStripSize(in);
+	tmsize_t bufsize  = TIFFStripSize(in);
 	unsigned char *buf = (unsigned char *)_TIFFmalloc(bufsize);
 
 	if (buf) {
 		tstrip_t s, ns = TIFFNumberOfStrips(in);
-		uint32 *bytecounts;
+		uint64_t *bytecounts;
 
 		if (!TIFFGetField(in, TIFFTAG_STRIPBYTECOUNTS, &bytecounts)) {
 			fprintf(stderr, "tiffsplit: strip byte counts are missing\n");
+                        _TIFFfree(buf);
 			return (0);
 		}
 		for (s = 0; s < ns; s++) {
-			if (bytecounts[s] > (uint32)bufsize) {
-				buf = (unsigned char *)_TIFFrealloc(buf, bytecounts[s]);
+			if (bytecounts[s] > (uint64_t)bufsize) {
+				buf = (unsigned char *)_TIFFrealloc(buf, (tmsize_t)bytecounts[s]);
 				if (!buf)
 					return (0);
-				bufsize = bytecounts[s];
+				bufsize = (tmsize_t)bytecounts[s];
 			}
-			if (TIFFReadRawStrip(in, s, buf, bytecounts[s]) < 0 ||
-			    TIFFWriteRawStrip(out, s, buf, bytecounts[s]) < 0) {
+			if (TIFFReadRawStrip(in, s, buf, (tmsize_t)bytecounts[s]) < 0 ||
+			    TIFFWriteRawStrip(out, s, buf, (tmsize_t)bytecounts[s]) < 0) {
 				_TIFFfree(buf);
 				return (0);
 			}
@@ -263,26 +272,27 @@ cpStrips(TIFF* in, TIFF* out)
 static int
 cpTiles(TIFF* in, TIFF* out)
 {
-	tsize_t bufsize = TIFFTileSize(in);
+	tmsize_t bufsize = TIFFTileSize(in);
 	unsigned char *buf = (unsigned char *)_TIFFmalloc(bufsize);
 
 	if (buf) {
 		ttile_t t, nt = TIFFNumberOfTiles(in);
-		uint32 *bytecounts;
+		uint64_t *bytecounts;
 
 		if (!TIFFGetField(in, TIFFTAG_TILEBYTECOUNTS, &bytecounts)) {
 			fprintf(stderr, "tiffsplit: tile byte counts are missing\n");
+                        _TIFFfree(buf);
 			return (0);
 		}
 		for (t = 0; t < nt; t++) {
-			if (bytecounts[t] > (uint32) bufsize) {
-				buf = (unsigned char *)_TIFFrealloc(buf, bytecounts[t]);
+			if (bytecounts[t] > (uint64_t) bufsize) {
+				buf = (unsigned char *)_TIFFrealloc(buf, (tmsize_t)bytecounts[t]);
 				if (!buf)
 					return (0);
-				bufsize = bytecounts[t];
+				bufsize = (tmsize_t)bytecounts[t];
 			}
-			if (TIFFReadRawTile(in, t, buf, bytecounts[t]) < 0 ||
-			    TIFFWriteRawTile(out, t, buf, bytecounts[t]) < 0) {
+			if (TIFFReadRawTile(in, t, buf, (tmsize_t)bytecounts[t]) < 0 ||
+			    TIFFWriteRawTile(out, t, buf, (tmsize_t)bytecounts[t]) < 0) {
 				_TIFFfree(buf);
 				return (0);
 			}
